==> LMSE.v1.1.zip: LMSE.m <==
function [LMSE_parameters] = LMSE(y,r,EC,EC_value,c,IC,search_space,constraints)
% Author: Scott Albert
% Laboratory for Computational Motor Control
% 01/18/2017
% This function minimizes a squared error cost function to fit a two-state
% model of learning. This form of model fitting is equivalent to Maximum
% Likelihood Estimation for a two-state model of learning, where noise is
% present only in the production of a movement, not in the planning of a
% movement.
% Inputs:
%      y: an array containing the behavior
%      r: an array containing the perturbations
%      EC: an array of binary (0 or 1) values that indicate if an
%          error-clamp trial occurred
%      EC_value: an array of real-valued numbers that corresponds to
%          value of the error during any error clamp trials
%      IC: the initial guess for the two-state model parameters
%      search_space: a matrix where the first column is the lower bound for
%          the two-state model parameters, and the second column is the
%          upper bound for the two-state model parameters
%      constraints: a 2x1 array where the first entry is a tolerance by
%          which aSlow exceeds aFast, and the second entry is a tolerance
%          by which bFast exceeds bSlow

function MSE = cost_fun(parameters)
    % gets the simulations y value
    ySim = two_state_simulation_without_noise(parameters,r,EC,EC_value,c);
    
    % compute the mean-squared error
    MSE = mean((y-ySim).^2);
end

% constraint matrices
%         a1    a2    b1    b2   x11   x12
A_con = [-1,    1,    0,    0,    0,    0;
          0,    0,    1,   -1,    0,    0];
b_con = [-constraints(1); % -a1 + a2 < -deltaA
         -constraints(2)]; % b1 - b2 < -deltaB
% lower and upper bounds
lb = search_space(:,1);
ub = search_space(:,2);

% makes sure that fmincon does not print results to screen
options = optimset('Display','off');

% uses fmincon for nonlinear minimization (uses the input parameter values
% as the initial guess)
LMSE_parameters = fmincon(@cost_fun, IC, A_con, b_con, [], [], lb, ...
    ub, [], options);

end

==> LMSE.v1.2.zip: LMSE.m <==
function [LMSE_parameters] = LMSE(y,r,EC,EC_value,SB,c,IC,search_space,constraints)
% Author: Scott Albert
% Laboratory for Computational Motor Control
% 01/18/2017
% This function minimizes a squared error cost function to fit a two-state
% model of learning. This form of model fitting is equivalent to Maximum
% Likelihood Estimation for a two-state model of learning, where noise is
% present only in the production of a movement, not in the planning of a
% movement.
% Inputs:
%      y: an array containing the behavior
%      r: an array containing the perturbations
%      EC: an array of binary (0 or 1) values that indicate if an
%          error-clamp trial occurred
%      EC_value: an array of real-valued numbers that corresponds to
%          value of the error during any error clamp trials
%      IC: the initial guess for the two-state model parameters
%      search_space: a matrix where the first column is the lower bound for
%          the two-state model parameters, and the second column is the
%          upper bound for the two-state model parameters
%      constraints: a 2x1 array where the first entry is a tolerance by
%          which aSlow exceeds aFast, and the second entry is a tolerance
%          by which bFast exceeds bSlow

function MSE = cost_fun(parameters)
    % gets the simulations y value
    ySim = two_state_simulation_without_noise(parameters,r,EC,EC_value,SB,c);
    
    % compute the mean-squared error
    MSE = mean((y-ySim).^2);
end

% constraint matrices
%         a1    a2    b1    b2   x11   x12   d
A_con = [-1,    1,    0,    0,    0,    0,   0;
          0,    0,    1,   -1,    0,    0,   0];
b_con = [-constraints(1); % -a1 + a2 < -deltaA
         -constraints(2)]; % b1 - b2 < -deltaB
% lower and upper bounds
lb = search_space(:,1);
ub = search_space(:,2);

% makes sure that fmincon does not print results to screen
options = optimset('Display','off');

% uses fmincon for nonlinear minimization (uses the input parameter values
% as the initial guess)
LMSE_parameters = fmincon(@cost_fun, IC, A_con, b_con, [], [], lb, ...
    ub, [], options);

end
